@@ -23,10 +23,8 @@
 #include <glib/gi18n.h>
 #include <libgnomeui/libgnomeui.h>
 #include "flegitaactionwidget.h"
-#if	ENABLE_PDF_OUTPUT
 #include <cairo.h>
 #include <cairo-pdf.h>
-#endif
 #include "flegitatypes.h"
 
 #ifdef HAVE_CONFIG_H
@@ -57,6 +55,12 @@ struct _FlegitaActionInfo {
   const gchar* 		filename;
   FlegitaFormat		format;
   gint			resolution;
+
+  gboolean		new_image; 	/* wether an incoming image is the
+					   first of an mass acquisition */
+  cairo_t		*cairo;
+  cairo_surface_t	*surface;
+  gchar			*realpath;
 };
 
 enum Column {
@@ -78,10 +82,17 @@ void				faw_action_selected				(GtkComboBox *combo,
 void				faw_format_selected				(GtkComboBox *combo,
 										 FlegitaActionWidget *faw);
 
+void				faw_acquisition_started				(GnomeScanContext* context,
+										 GnomeScanForecast *forecast,
+										 FlegitaActionWidget *faw);
+
 void				faw_image_acquired				(GnomeScanContext *context,
 										 GnomeScanResult *acquisition,
 										 FlegitaActionWidget *faw);
 
+void				faw_acquisition_terminated			(GnomeScanContext *context,
+										 FlegitaActionWidget *faw);
+
 void				faw_dir_selection_changed			(GtkFileChooser *chooser,
 										 FlegitaActionWidget *faw);
 
@@ -110,11 +121,9 @@ void				faw_write_image					(GdkPixbuf *pixbuf,
 										 gchar *filename,
 										 FlegitaFormat format);
 
-#if	ENABLE_PDF_OUTPUT
 void				faw_write_pdf					(FlegitaActionInfo *info,
 										 GdkPixbuf *pixbuf,
 										 gchar *filename);
-#endif
 
 
 /********************************
@@ -320,6 +329,16 @@ flegita_action_widget_new (GnomeScanContext *context)
 		    (GCallback) faw_image_acquired,
 		    faw);
 
+  g_signal_connect (context,
+		    "acquisition-started",
+		    (GCallback) faw_acquisition_started,
+		    faw);
+
+  g_signal_connect (context,
+		    "acquisition-terminated",
+		    (GCallback) faw_acquisition_terminated,
+		    faw);
+
   /* preselect */
   gtk_combo_box_set_active (GTK_COMBO_BOX (priv->action), 0);
 
@@ -385,23 +404,27 @@ faw_format_selected (GtkComboBox *combo,
   case FORMAT_PNG:
   case FORMAT_JPEG:
   case FORMAT_TIFF:
-#ifdef	GTK_210
+  case FORMAT_PDF:		/* For now, we create PDF for screen, not print */
     info->resolution = gdk_screen_get_resolution (gdk_screen_get_default ());
-#else 
-    info->resolution = 96.;
-#endif
     break;
-#if ENABLE_PDF_OUTPUT
-  case FORMAT_PDF:
-    /* TODO: check correct resolution for PDF (ready to print with nice result) */
-    info->resolution = 300;
-    break;
-#endif
+/*   case FORMAT_PDF: */
+/*     /\* TODO: check correct resolution for PDF (ready to print with nice result) *\/ */
+/*     info->resolution = 300; */
+/*     break; */
   }
 
   gnome_scan_context_set_resolution (faw->context, info->resolution);
 }
 
+void
+faw_acquisition_started (GnomeScanContext *context,
+			 GnomeScanForecast *forecast,
+			 FlegitaActionWidget *faw)
+{
+  FlegitaActionWidgetPrivate *priv = GET_PRIVATE (faw);
+  priv->info->new_image = TRUE;
+}
+
 void
 faw_image_acquired (GnomeScanContext *context,
 		    GnomeScanResult *acquisition,
@@ -416,6 +439,24 @@ faw_image_acquired (GnomeScanContext *context,
   }
 }
 
+void
+faw_acquisition_terminated (GnomeScanContext *context,
+			    FlegitaActionWidget *faw)
+{
+  FlegitaActionWidgetPrivate *priv = GET_PRIVATE (faw);
+
+  if (priv->info->cairo) {
+    cairo_destroy (priv->info->cairo);
+    priv->info->cairo = NULL;
+    cairo_surface_destroy (priv->info->surface);
+    priv->info->surface = NULL;
+    g_free (priv->info->realpath);
+    priv->info->realpath = NULL;
+  }
+}
+
+
+
 /********************************
  * 	      OTHER		*
  ********************************/
@@ -487,7 +528,6 @@ GtkWidget*
 faw_create_dir_selector (FlegitaActionWidget *faw)
 {
   GtkWidget *box, *widget;
-  gchar *dir;
 
   box = gtk_hbox_new (FALSE, 6);
   /* translators, label "in" before [dir] selector */
@@ -505,9 +545,7 @@ faw_create_dir_selector (FlegitaActionWidget *faw)
 
   /* default value */
   /* TODO: store last dir */
-  dir = g_build_filename (g_get_home_dir (), "Documents", NULL);
-  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget), dir);
-  g_free (dir);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget), g_get_home_dir ());
 
   return box;
 }
@@ -537,10 +575,9 @@ faw_create_file_selector (FlegitaActionWidget *faw)
 
   /* TODO: store laste filename */
   /* translators, default proposition for filename. */
-  gtk_entry_set_text (GTK_ENTRY (widget), _("Image"));
+  gtk_entry_set_text (GTK_ENTRY (widget), _("Scanned picture"));
 
   gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
-
   gtk_box_pack_start (GTK_BOX (box), gtk_label_new_with_mnemonic ("."), FALSE, TRUE, 0);
 
   /* FORMAT */
@@ -566,7 +603,7 @@ faw_create_file_selector (FlegitaActionWidget *faw)
   		    (GCallback) faw_format_selected,
   		    faw);
 
-  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 3);
 
   gtk_box_pack_start (GTK_BOX (box), widget, FALSE, TRUE, 0);
 
@@ -584,14 +621,12 @@ faw_file_selector_fill (FlegitaFormat format,
     "stock_save-pdf"
   };
 
-  if (ENABLE_PDF_OUTPUT || format != FORMAT_PDF) {
-    GtkTreeIter iter;
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter,
-			COLUMN_ICON, icons[format],
-			COLUMN_LABEL, gs_enum_get_nick (FLEGITA_TYPE_FORMAT, format),
-			-1);
-  }
+  GtkTreeIter iter;
+  gtk_list_store_append (store, &iter);
+  gtk_list_store_set (store, &iter,
+		      COLUMN_ICON, icons[format],
+		      COLUMN_LABEL, gs_enum_get_nick (FLEGITA_TYPE_FORMAT, format),
+		      -1);
 }
 
 void
@@ -614,6 +649,7 @@ faw_action_save (GnomeScanResult *acquisition,
 
     if (filename)
       g_free (filename);
+
     filename = g_build_filename (info->dir, basename, NULL);
 
     n++;
@@ -624,15 +660,13 @@ faw_action_save (GnomeScanResult *acquisition,
   case FORMAT_JPEG:
   case FORMAT_TIFF:
     faw_write_image (acquisition->image, filename, info->format);
+    g_free (filename);
     break;
-#if	ENABLE_PDF_OUTPUT
   case FORMAT_PDF:
-    faw_write_pdf (info, acquisition->image, filename);
-#endif
+    info->realpath = info->realpath ? info->realpath : filename;
+    faw_write_pdf (info, acquisition->image, info->realpath);
     break;
   }
-
-  g_free (filename);
 }
 
 /* maybe the simplest backend for so many format :) */
@@ -645,7 +679,6 @@ faw_write_image (GdkPixbuf *pixbuf,
   gdk_pixbuf_save (pixbuf, filename, gs_enum_get_nick (FLEGITA_TYPE_FORMAT, format), NULL, NULL);
 }
 
-#if	ENABLE_PDF_OUTPUT
 void
 faw_write_pdf (FlegitaActionInfo *info,
 	       GdkPixbuf *pixbuf,
@@ -654,13 +687,21 @@ faw_write_pdf (FlegitaActionInfo *info,
   cairo_t *cr;
   cairo_surface_t *cs;
 
-  cs = cairo_pdf_surface_create (filename, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf));
-  cairo_surface_set_fallback_resolution (cs,(gint) info->resolution, (gint) info->resolution);
-  cr = cairo_create (cs);
+  g_debug ("Writing to %s", info->realpath);
+  if (info->new_image) {
+    info->surface = cs = cairo_pdf_surface_create (info->realpath, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf));
+    cairo_surface_set_fallback_resolution (cs,(gint) info->resolution, (gint) info->resolution);
+    info->cairo = cr = cairo_create (cs);
+    info->new_image = FALSE;
+  }
+  else {
+    cr = info->cairo;
+    cs = info->surface;
+  }
+
+  /* paint the image in a page */
   gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
   cairo_paint (cr);
   cairo_show_page (cr);
-  cairo_destroy (cr);
-  cairo_surface_destroy (cs);
+/*   cairo_surface_flush (cs); */
 }
-#endif
